@@ -26,10 +26,11 @@
 #endif
 
 #include <gst/gst.h>
-#include <gst/control/control.h>
+#include <gst/controller/gstcontroller.h>
 
 #include "gst/gst-i18n-app.h"
 
+#include <stdlib.h>
 #include <string.h>
 #include <locale.h>
 #include <glib/gprintf.h>
@@ -54,7 +55,7 @@ n_print (const char *format, ...)
 }
 
 static gboolean
-print_field (GQuark field, GValue * value, gpointer pfx)
+print_field (GQuark field, const GValue * value, gpointer pfx)
 {
   gchar *str = gst_value_serialize (value);
 
@@ -87,6 +88,7 @@ print_caps (const GstCaps * caps, const gchar * pfx)
   }
 }
 
+#if 0
 static void
 print_formats (const GstFormat * formats)
 {
@@ -103,6 +105,7 @@ print_formats (const GstFormat * formats)
     formats++;
   }
 }
+#endif
 
 static void
 print_query_types (const GstQueryType * types)
@@ -122,6 +125,7 @@ print_query_types (const GstQueryType * types)
 }
 
 #ifndef GST_DISABLE_ENUMTYPES
+#if 0
 static void
 print_event_masks (const GstEventMask * masks)
 {
@@ -169,6 +173,7 @@ print_event_masks (const GstEventMask * masks)
     masks++;
   }
 }
+#endif
 #else
 static void
 print_event_masks (const GstEventMask * masks)
@@ -235,13 +240,33 @@ print_hierarchy (GType type, gint level, gint * maxlevel)
     n_print ("\n");
 }
 
+static void
+print_interfaces (GType type)
+{
+  guint n_ifaces;
+  GType *iface, *ifaces = g_type_interfaces (type, &n_ifaces);
+
+  if (ifaces) {
+    if (n_ifaces) {
+      g_print ("%sImplemented Interfaces:\n", _name);
+      iface = ifaces;
+      while (*iface) {
+        g_print ("%s  %s\n", _name, g_type_name (*iface));
+        iface++;
+      }
+      g_print ("%s\n", _name);
+      g_free (ifaces);
+    }
+  }
+}
+
 static void
 print_element_properties_info (GstElement * element)
 {
   GParamSpec **property_specs;
-  gint num_properties, i;
+  guint num_properties, i;
   gboolean readable;
-  const char *string_val;
+  gboolean first_flag;
 
   property_specs = g_object_class_list_properties
       (G_OBJECT_GET_CLASS (element), &num_properties);
@@ -255,39 +280,70 @@ print_element_properties_info (GstElement * element)
     readable = FALSE;
 
     g_value_init (&value, param->value_type);
-    if (param->flags & G_PARAM_READABLE) {
-      g_object_get_property (G_OBJECT (element), param->name, &value);
-      readable = TRUE;
-    }
 
     n_print ("  %-20s: %s\n", g_param_spec_get_name (param),
         g_param_spec_get_blurb (param));
 
+    first_flag = TRUE;
+    n_print ("%-23.23s flags: ", "");
+    if (param->flags & G_PARAM_READABLE) {
+      g_object_get_property (G_OBJECT (element), param->name, &value);
+      readable = TRUE;
+      g_print ((first_flag ? "readable" : ", readable"));
+      first_flag = FALSE;
+    }
+    if (param->flags & G_PARAM_WRITABLE) {
+      g_print ((first_flag ? "writable" : ", writable"));
+      first_flag = FALSE;
+    }
+    if (param->flags & GST_PARAM_CONTROLLABLE) {
+      g_print ((first_flag ? "controllable" : ", controllable"));
+      first_flag = FALSE;
+    }
+    n_print ("\n");
+
     switch (G_VALUE_TYPE (&value)) {
       case G_TYPE_STRING:
-        string_val = g_value_get_string (&value);
+      {
+        GParamSpecString *pstring = G_PARAM_SPEC_STRING (param);
+
         n_print ("%-23.23s String. ", "");
+
+        if (pstring->default_value == NULL)
+          g_print ("Default: null ");
+        else
+          g_print ("Default: \"%s\" ", pstring->default_value);
+
         if (readable) {
+          const char *string_val = g_value_get_string (&value);
+
           if (string_val == NULL)
-            g_print ("(Default \"\")");
+            g_print ("Current: null");
           else
-            g_print ("(Default \"%s\")", g_value_get_string (&value));
+            g_print ("Current: \"%s\"", string_val);
         }
         break;
+      }
       case G_TYPE_BOOLEAN:
+      {
+        GParamSpecBoolean *pboolean = G_PARAM_SPEC_BOOLEAN (param);
+
         n_print ("%-23.23s Boolean. ", "");
+        g_print ("Default: %s ", (pboolean->default_value ? "true" : "false"));
         if (readable)
-          g_print ("(Default %s)",
+          g_print ("Current: %s",
               (g_value_get_boolean (&value) ? "true" : "false"));
         break;
+      }
       case G_TYPE_ULONG:
       {
         GParamSpecULong *pulong = G_PARAM_SPEC_ULONG (param);
 
         n_print ("%-23.23s Unsigned Long. ", "");
+        g_print ("Range: %lu - %lu Default: %lu ",
+            pulong->minimum, pulong->maximum, pulong->default_value);
         if (readable)
-          g_print ("Range: %lu - %lu (Default %lu)",
-              pulong->minimum, pulong->maximum, g_value_get_ulong (&value));
+          g_print ("Current: %lu", g_value_get_ulong (&value));
         break;
       }
       case G_TYPE_LONG:
@@ -295,9 +351,10 @@ print_element_properties_info (GstElement * element)
         GParamSpecLong *plong = G_PARAM_SPEC_LONG (param);
 
         n_print ("%-23.23s Long. ", "");
+        g_print ("Range: %ld - %ld Default: %ld ",
+            plong->minimum, plong->maximum, plong->default_value);
         if (readable)
-          g_print ("Range: %ld - %ld (Default %ld)",
-              plong->minimum, plong->maximum, g_value_get_long (&value));
+          g_print ("Current: %ld", g_value_get_long (&value));
         break;
       }
       case G_TYPE_UINT:
@@ -305,9 +362,10 @@ print_element_properties_info (GstElement * element)
         GParamSpecUInt *puint = G_PARAM_SPEC_UINT (param);
 
         n_print ("%-23.23s Unsigned Integer. ", "");
+        g_print ("Range: %u - %u Default: %u ",
+            puint->minimum, puint->maximum, puint->default_value);
         if (readable)
-          g_print ("Range: %u - %u (Default %u)",
-              puint->minimum, puint->maximum, g_value_get_uint (&value));
+          g_print ("Current: %u", g_value_get_uint (&value));
         break;
       }
       case G_TYPE_INT:
@@ -315,9 +373,10 @@ print_element_properties_info (GstElement * element)
         GParamSpecInt *pint = G_PARAM_SPEC_INT (param);
 
         n_print ("%-23.23s Integer. ", "");
+        g_print ("Range: %d - %d Default: %d ",
+            pint->minimum, pint->maximum, pint->default_value);
         if (readable)
-          g_print ("Range: %d - %d (Default %d)",
-              pint->minimum, pint->maximum, g_value_get_int (&value));
+          g_print ("Current: %d", g_value_get_int (&value));
         break;
       }
       case G_TYPE_UINT64:
@@ -325,10 +384,11 @@ print_element_properties_info (GstElement * element)
         GParamSpecUInt64 *puint64 = G_PARAM_SPEC_UINT64 (param);
 
         n_print ("%-23.23s Unsigned Integer64. ", "");
+        g_print ("Range: %" G_GUINT64_FORMAT " - %" G_GUINT64_FORMAT
+            " Default: %" G_GUINT64_FORMAT " ",
+            puint64->minimum, puint64->maximum, puint64->default_value);
         if (readable)
-          g_print ("Range: %" G_GUINT64_FORMAT " - %"
-              G_GUINT64_FORMAT " (Default %" G_GUINT64_FORMAT ")",
-              puint64->minimum, puint64->maximum, g_value_get_uint64 (&value));
+          g_print ("Current: %" G_GUINT64_FORMAT, g_value_get_uint64 (&value));
         break;
       }
       case G_TYPE_INT64:
@@ -336,36 +396,36 @@ print_element_properties_info (GstElement * element)
         GParamSpecInt64 *pint64 = G_PARAM_SPEC_INT64 (param);
 
         n_print ("%-23.23s Integer64. ", "");
+        g_print ("Range: %" G_GINT64_FORMAT " - %" G_GINT64_FORMAT
+            " Default: %" G_GINT64_FORMAT " ",
+            pint64->minimum, pint64->maximum, pint64->default_value);
         if (readable)
-          g_print ("Range: %" G_GINT64_FORMAT " - %" G_GINT64_FORMAT
-              " (Default %" G_GINT64_FORMAT ")", pint64->minimum,
-              pint64->maximum, g_value_get_int64 (&value));
+          g_print ("Current: %" G_GINT64_FORMAT, g_value_get_int64 (&value));
         break;
       }
       case G_TYPE_FLOAT:
       {
         GParamSpecFloat *pfloat = G_PARAM_SPEC_FLOAT (param);
 
-        n_print ("%-23.23s Float. Default: %-8.8s %15.7g\n", "", "",
-            g_value_get_float (&value));
-        n_print ("%-23.23s Range: %15.7g - %15.7g", "",
-            pfloat->minimum, pfloat->maximum);
+        n_print ("%-23.23s Float. ", "");
+        g_print ("Range: %15.7g - %15.7g Default: %15.7g ",
+            pfloat->minimum, pfloat->maximum, pfloat->default_value);
+        if (readable)
+          g_print ("Current: %15.7g", g_value_get_float (&value));
         break;
       }
       case G_TYPE_DOUBLE:
       {
         GParamSpecDouble *pdouble = G_PARAM_SPEC_DOUBLE (param);
 
-        n_print ("%-23.23s Double. Default: %-8.8s %15.7g\n", "", "",
-            g_value_get_double (&value));
-        n_print ("%-23.23s Range: %15.7g - %15.7g", "",
-            pdouble->minimum, pdouble->maximum);
+        n_print ("%-23.23s Double. ", "");
+        g_print ("Range: %15.7g - %15.7g Default: %15.7g ",
+            pdouble->minimum, pdouble->maximum, pdouble->default_value);
+        if (readable)
+          g_print ("Current: %15.7g", g_value_get_double (&value));
         break;
       }
       default:
-        if (param->value_type == GST_TYPE_URI) {
-          n_print ("%-23.23s URI", "");
-        }
         if (param->value_type == GST_TYPE_CAPS) {
           const GstCaps *caps = gst_value_get_caps (&value);
 
@@ -388,14 +448,15 @@ print_element_properties_info (GstElement * element)
             j++;
           }
 
-          n_print ("%-23.23s Enum \"%s\" (default %d, \"%s\")", "",
+          n_print ("%-23.23s Enum \"%s\" Current: %d, \"%s\"", "",
               g_type_name (G_VALUE_TYPE (&value)),
               enum_value, values[j].value_nick);
 
           j = 0;
           while (values[j].value_name) {
-            g_print ("\n%s%-23.23s    (%d): \t%s", "",
-                _name, values[j].value, values[j].value_nick);
+            g_print ("\n%s%-23.23s    %d) %-16s - %s", "",
+                _name, values[j].value, values[j].value_nick,
+                values[j].value_name);
             j++;
           }
           /* g_type_class_unref (ec); */
@@ -419,7 +480,7 @@ print_element_properties_info (GstElement * element)
             j++;
           }
 
-          n_print ("%-23.23s Flags \"%s\" (default %d, \"%s\")", "",
+          n_print ("%-23.23s Flags \"%s\" Current: %d, \"%s\"", "",
               g_type_name (G_VALUE_TYPE (&value)),
               flags_value, (flags ? flags->str : "(none)"));
 
@@ -435,6 +496,16 @@ print_element_properties_info (GstElement * element)
         } else if (G_IS_PARAM_SPEC_OBJECT (param)) {
           n_print ("%-23.23s Object of type \"%s\"", "",
               g_type_name (param->value_type));
+        } else if (G_IS_PARAM_SPEC_BOXED (param)) {
+          n_print ("%-23.23s Boxed pointer of type \"%s\"", "",
+              g_type_name (param->value_type));
+        } else if (G_IS_PARAM_SPEC_POINTER (param)) {
+          if (param->value_type != G_TYPE_POINTER) {
+            n_print ("%-23.23s Pointer of type \"%s\".", "",
+                g_type_name (param->value_type));
+          } else {
+            n_print ("%-23.23s Pointer.", "");
+          }
         } else {
           n_print ("%-23.23s Unknown type %ld \"%s\"", "", param->value_type,
               g_type_name (param->value_type));
@@ -448,6 +519,8 @@ print_element_properties_info (GstElement * element)
   }
   if (num_properties == 0)
     n_print ("  none\n");
+
+  g_free (property_specs);
 }
 
 static void
@@ -455,7 +528,7 @@ print_pad_templates_info (GstElement * element, GstElementFactory * factory)
 {
   GstElementClass *gstelement_class;
   const GList *pads;
-  GstPadTemplate *padtemplate;
+  GstStaticPadTemplate *padtemplate;
 
   n_print ("Pad Templates:\n");
   if (!factory->numpadtemplates) {
@@ -465,9 +538,9 @@ print_pad_templates_info (GstElement * element, GstElementFactory * factory)
 
   gstelement_class = GST_ELEMENT_CLASS (G_OBJECT_GET_CLASS (element));
 
-  pads = factory->padtemplates;
+  pads = factory->staticpadtemplates;
   while (pads) {
-    padtemplate = (GstPadTemplate *) (pads->data);
+    padtemplate = (GstStaticPadTemplate *) (pads->data);
     pads = g_list_next (pads);
 
     if (padtemplate->direction == GST_PAD_SRC)
@@ -488,9 +561,9 @@ print_pad_templates_info (GstElement * element, GstElementFactory * factory)
     } else
       n_print ("    Availability: UNKNOWN!!!\n");
 
-    if (padtemplate->caps) {
+    if (padtemplate->static_caps.string) {
       n_print ("    Capabilities:\n");
-      print_caps (padtemplate->caps, "      ");
+      print_caps (gst_static_caps_get (&padtemplate->static_caps), "      ");
     }
 
     n_print ("\n");
@@ -505,40 +578,12 @@ print_element_flag_info (GstElement * element)
   n_print ("\n");
   n_print ("Element Flags:\n");
 
-  if (GST_FLAG_IS_SET (element, GST_ELEMENT_COMPLEX)) {
-    n_print ("  GST_ELEMENT_COMPLEX\n");
-    have_flags = TRUE;
-  }
-  if (GST_FLAG_IS_SET (element, GST_ELEMENT_DECOUPLED)) {
-    n_print ("  GST_ELEMENT_DECOUPLED\n");
-    have_flags = TRUE;
-  }
-  if (GST_FLAG_IS_SET (element, GST_ELEMENT_THREAD_SUGGESTED)) {
-    n_print ("  GST_ELEMENT_THREADSUGGESTED\n");
-    have_flags = TRUE;
-  }
-  if (GST_FLAG_IS_SET (element, GST_ELEMENT_EVENT_AWARE)) {
-    n_print ("  GST_ELEMENT_EVENT_AWARE\n");
-    have_flags = TRUE;
-  }
   if (!have_flags)
     n_print ("  no flags set\n");
 
   if (GST_IS_BIN (element)) {
     n_print ("\n");
     n_print ("Bin Flags:\n");
-    if (GST_FLAG_IS_SET (element, GST_BIN_FLAG_MANAGER)) {
-      n_print ("  GST_BIN_FLAG_MANAGER\n");
-      have_flags = TRUE;
-    }
-    if (GST_FLAG_IS_SET (element, GST_BIN_SELF_SCHEDULABLE)) {
-      n_print ("  GST_BIN_SELF_SCHEDULABLE\n");
-      have_flags = TRUE;
-    }
-    if (GST_FLAG_IS_SET (element, GST_BIN_FLAG_PREFER_COTHREADS)) {
-      n_print ("  GST_BIN_FLAG_PREFER_COTHREADS\n");
-      have_flags = TRUE;
-    }
     if (!have_flags)
       n_print ("  no flags set\n");
   }
@@ -556,11 +601,7 @@ print_implementation_info (GstElement * element)
   n_print ("\n");
   n_print ("Element Implementation:\n");
 
-  if (element->loopfunc)
-    n_print ("  loopfunc()-based element: %s\n",
-        GST_DEBUG_FUNCPTR_NAME (element->loopfunc));
-  else
-    n_print ("  No loopfunc(), must be chain-based or not configured yet\n");
+  n_print ("  No loopfunc(), must be chain-based or not configured yet\n");
 
   n_print ("  Has change_state() function: %s\n",
       GST_DEBUG_FUNCPTR_NAME (gstelement_class->change_state));
@@ -625,7 +666,6 @@ print_pad_info (GstElement * element)
 {
   const GList *pads;
   GstPad *pad;
-  GstRealPad *realpad;
 
   n_print ("\n");
   n_print ("Pads:\n");
@@ -635,128 +675,58 @@ print_pad_info (GstElement * element)
     return;
   }
 
-  pads = gst_element_get_pad_list (element);
+  pads = element->pads;
   while (pads) {
     pad = GST_PAD (pads->data);
     pads = g_list_next (pads);
-    realpad = GST_PAD_REALIZE (pad);
 
     n_print ("");
 
-    if (gst_pad_get_direction (GST_PAD (realpad)) == GST_PAD_SRC)
+    if (gst_pad_get_direction (pad) == GST_PAD_SRC)
       g_print ("  SRC: '%s'", gst_pad_get_name (pad));
-    else if (gst_pad_get_direction (GST_PAD (realpad)) == GST_PAD_SINK)
+    else if (gst_pad_get_direction (pad) == GST_PAD_SINK)
       g_print ("  SINK: '%s'", gst_pad_get_name (pad));
     else
       g_print ("  UNKNOWN!!!: '%s'", gst_pad_get_name (pad));
 
-    if (GST_IS_GHOST_PAD (pad))
-      g_print (", ghost of real pad %s:%s\n", GST_DEBUG_PAD_NAME (realpad));
-    else
-      g_print ("\n");
+    g_print ("\n");
 
     n_print ("    Implementation:\n");
-    if (realpad->chainfunc)
+    if (pad->chainfunc)
       n_print ("      Has chainfunc(): %s\n",
-          GST_DEBUG_FUNCPTR_NAME (realpad->chainfunc));
-    if (realpad->getfunc)
-      n_print ("      Has getfunc(): %s\n",
-          GST_DEBUG_FUNCPTR_NAME (realpad->getfunc));
-    if (realpad->formatsfunc != gst_pad_get_formats_default) {
-      n_print ("      Supports seeking/conversion/query formats:\n");
-      print_formats (gst_pad_get_formats (GST_PAD (realpad)));
-    }
-    if (realpad->convertfunc != gst_pad_convert_default)
-      n_print ("      Has custom convertfunc(): %s\n",
-          GST_DEBUG_FUNCPTR_NAME (realpad->convertfunc));
-    if (realpad->eventfunc != gst_pad_event_default)
+          GST_DEBUG_FUNCPTR_NAME (pad->chainfunc));
+    if (pad->getrangefunc)
+      n_print ("      Has getrangefunc(): %s\n",
+          GST_DEBUG_FUNCPTR_NAME (pad->getrangefunc));
+    if (pad->eventfunc != gst_pad_event_default)
       n_print ("      Has custom eventfunc(): %s\n",
-          GST_DEBUG_FUNCPTR_NAME (realpad->eventfunc));
-    if (realpad->eventmaskfunc != gst_pad_get_event_masks_default) {
-      n_print ("        Provides event masks:\n");
-      print_event_masks (gst_pad_get_event_masks (GST_PAD (realpad)));
-    }
-    if (realpad->queryfunc != gst_pad_query_default)
+          GST_DEBUG_FUNCPTR_NAME (pad->eventfunc));
+    if (pad->queryfunc != gst_pad_query_default)
       n_print ("      Has custom queryfunc(): %s\n",
-          GST_DEBUG_FUNCPTR_NAME (realpad->queryfunc));
-    if (realpad->querytypefunc != gst_pad_get_query_types_default) {
+          GST_DEBUG_FUNCPTR_NAME (pad->queryfunc));
+    if (pad->querytypefunc != gst_pad_get_query_types_default) {
       n_print ("        Provides query types:\n");
-      print_query_types (gst_pad_get_query_types (GST_PAD (realpad)));
+      print_query_types (gst_pad_get_query_types (pad));
     }
 
-    if (realpad->intlinkfunc != gst_pad_get_internal_links_default)
+    if (pad->intlinkfunc != gst_pad_get_internal_links_default)
       n_print ("      Has custom intconnfunc(): %s\n",
-          GST_DEBUG_FUNCPTR_NAME (realpad->intlinkfunc));
+          GST_DEBUG_FUNCPTR_NAME (pad->intlinkfunc));
 
-    if (realpad->bufferallocfunc)
+    if (pad->bufferallocfunc)
       n_print ("      Has bufferallocfunc(): %s\n",
-          GST_DEBUG_FUNCPTR_NAME (realpad->bufferallocfunc));
+          GST_DEBUG_FUNCPTR_NAME (pad->bufferallocfunc));
 
     if (pad->padtemplate)
       n_print ("    Pad Template: '%s'\n", pad->padtemplate->name_template);
 
-    if (realpad->caps) {
+    if (pad->caps) {
       n_print ("    Capabilities:\n");
-      print_caps (realpad->caps, "      ");
+      print_caps (pad->caps, "      ");
     }
   }
 }
 
-static void
-print_dynamic_parameters_info (GstElement * element)
-{
-  GstDParamManager *dpman;
-  GParamSpec **specs = NULL;
-  gint x;
-
-  if ((dpman = gst_dpman_get_manager (element))) {
-    specs = gst_dpman_list_dparam_specs (dpman);
-  }
-
-  if (specs && specs[0] != NULL) {
-    n_print ("\n");
-    n_print ("Dynamic Parameters:\n");
-
-    for (x = 0; specs[x] != NULL; x++) {
-      g_print ("  %-20.20s: ", g_param_spec_get_name (specs[x]));
-
-      switch (G_PARAM_SPEC_VALUE_TYPE (specs[x])) {
-        case G_TYPE_INT64:
-          g_print ("64 Bit Integer (Default %" G_GINT64_FORMAT ", Range %"
-              G_GINT64_FORMAT " -> %" G_GINT64_FORMAT ")",
-              ((GParamSpecInt64 *) specs[x])->default_value,
-              ((GParamSpecInt64 *) specs[x])->minimum,
-              ((GParamSpecInt64 *) specs[x])->maximum);
-          break;
-        case G_TYPE_INT:
-          g_print ("Integer (Default %d, Range %d -> %d)",
-              ((GParamSpecInt *) specs[x])->default_value,
-              ((GParamSpecInt *) specs[x])->minimum,
-              ((GParamSpecInt *) specs[x])->maximum);
-          break;
-        case G_TYPE_FLOAT:
-          g_print ("Float. Default: %-8.8s %15.7g\n", "",
-              ((GParamSpecFloat *) specs[x])->default_value);
-          g_print ("%-23.23s Range: %15.7g - %15.7g", "",
-              ((GParamSpecFloat *) specs[x])->minimum,
-              ((GParamSpecFloat *) specs[x])->maximum);
-          break;
-        case G_TYPE_DOUBLE:
-          g_print ("Double. Default: %-8.8s %15.7g\n", "",
-              ((GParamSpecDouble *) specs[x])->default_value);
-          g_print ("%-23.23s Range: %15.7g - %15.7g", "",
-              ((GParamSpecDouble *) specs[x])->minimum,
-              ((GParamSpecDouble *) specs[x])->maximum);
-          break;
-        default:
-          g_print ("unknown %ld", G_PARAM_SPEC_VALUE_TYPE (specs[x]));
-      }
-      g_print ("\n");
-    }
-    g_free (specs);
-  }
-}
-
 #if 0
 static gint
 compare_signal_names (GSignalQuery * a, GSignalQuery * b)
@@ -854,7 +824,7 @@ print_children_info (GstElement * element)
   if (!GST_IS_BIN (element))
     return;
 
-  children = (GList *) gst_bin_get_list (GST_BIN (element));
+  children = (GList *) GST_BIN (element)->children;
   if (children) {
     n_print ("\n");
     g_print ("Children:\n");
@@ -869,21 +839,26 @@ print_children_info (GstElement * element)
 static void
 print_element_list (gboolean print_all)
 {
-  GList *plugins;
+  int plugincount = 0, featurecount = 0;
+  GList *plugins, *orig_plugins;
 
-  plugins = gst_registry_pool_plugin_list ();
+  orig_plugins = plugins = gst_default_registry_get_plugin_list ();
   while (plugins) {
-    GList *features;
+    GList *features, *orig_features;
     GstPlugin *plugin;
 
     plugin = (GstPlugin *) (plugins->data);
     plugins = g_list_next (plugins);
+    plugincount++;
 
-    features = gst_plugin_get_feature_list (plugin);
+    orig_features = features =
+        gst_registry_get_feature_list_by_plugin (gst_registry_get_default (),
+        plugin->desc.name);
     while (features) {
       GstPluginFeature *feature;
 
       feature = GST_PLUGIN_FEATURE (features->data);
+      featurecount++;
 
       if (GST_IS_ELEMENT_FACTORY (feature)) {
         GstElementFactory *factory;
@@ -926,13 +901,6 @@ print_element_list (gboolean print_all)
           if (!print_all)
             g_print ("no extensions\n");
         }
-      } else if (GST_IS_SCHEDULER_FACTORY (feature)) {
-        GstSchedulerFactory *factory;
-
-        factory = GST_SCHEDULER_FACTORY (feature);
-        if (!print_all)
-          g_print ("%s:  %s: %s\n", plugin->desc.name,
-              GST_PLUGIN_FEATURE_NAME (factory), factory->longdesc);
       } else {
         if (!print_all)
           n_print ("%s:  %s (%s)\n", plugin->desc.name,
@@ -942,20 +910,29 @@ print_element_list (gboolean print_all)
 
       features = g_list_next (features);
     }
+
+    gst_plugin_feature_list_free (orig_features);
   }
+
+  gst_plugin_list_free (plugins);
+
+  g_print ("\nTotal plugins: %d\nTotal features: %d\n",
+      plugincount, featurecount);
 }
 
 static void
 print_plugin_info (GstPlugin * plugin)
 {
   n_print ("Plugin Details:\n");
-  n_print ("  Name:\t\t%s\n", plugin->desc.name);
-  n_print ("  Description:\t%s\n", plugin->desc.description);
-  n_print ("  Filename:\t%s\n", plugin->filename ? plugin->filename : "(null)");
-  n_print ("  Version:\t%s\n", plugin->desc.version);
-  n_print ("  License:\t%s\n", plugin->desc.license);
-  n_print ("  Package:\t%s\n", plugin->desc.package);
-  n_print ("  Origin URL:\t%s\n", plugin->desc.origin);
+  n_print ("  Name:\t\t\t%s\n", plugin->desc.name);
+  n_print ("  Description:\t\t%s\n", plugin->desc.description);
+  n_print ("  Filename:\t\t%s\n",
+      plugin->filename ? plugin->filename : "(null)");
+  n_print ("  Version:\t\t%s\n", plugin->desc.version);
+  n_print ("  License:\t\t%s\n", plugin->desc.license);
+  n_print ("  Source module:\t%s\n", plugin->desc.source);
+  n_print ("  Binary package:\t%s\n", plugin->desc.package);
+  n_print ("  Origin URL:\t\t%s\n", plugin->desc.origin);
   n_print ("\n");
 }
 
@@ -966,11 +943,12 @@ print_plugin_features (GstPlugin * plugin)
   gint num_features = 0;
   gint num_elements = 0;
   gint num_types = 0;
-  gint num_schedulers = 0;
   gint num_indexes = 0;
   gint num_other = 0;
 
-  features = gst_plugin_get_feature_list (plugin);
+  features =
+      gst_registry_get_feature_list_by_plugin (gst_registry_get_default (),
+      plugin->desc.name);
 
   while (features) {
     GstPluginFeature *feature;
@@ -981,7 +959,7 @@ print_plugin_features (GstPlugin * plugin)
       GstElementFactory *factory;
 
       factory = GST_ELEMENT_FACTORY (feature);
-      n_print ("  %s: %s\n", GST_OBJECT_NAME (factory),
+      n_print ("  %s: %s\n", GST_PLUGIN_FEATURE_NAME (factory),
           factory->details.longname);
       num_elements++;
     }
@@ -1011,12 +989,6 @@ print_plugin_features (GstPlugin * plugin)
         g_print ("%s type: N/A\n", plugin->desc.name);
 
       num_types++;
-    } else if (GST_IS_SCHEDULER_FACTORY (feature)) {
-      GstSchedulerFactory *factory;
-
-      factory = GST_SCHEDULER_FACTORY (feature);
-      n_print ("  %s: %s\n", GST_OBJECT_NAME (factory), factory->longdesc);
-      num_schedulers++;
     } else {
       n_print ("  %s (%s)\n", gst_object_get_name (GST_OBJECT (feature)),
           g_type_name (G_OBJECT_TYPE (feature)));
@@ -1031,8 +1003,6 @@ print_plugin_features (GstPlugin * plugin)
     n_print ("  +-- %d elements\n", num_elements);
   if (num_types > 0)
     n_print ("  +-- %d types\n", num_types);
-  if (num_schedulers > 0)
-    n_print ("  +-- %d schedulers\n", num_schedulers);
   if (num_indexes > 0)
     n_print ("  +-- %d indexes\n", num_indexes);
   if (num_other > 0)
@@ -1047,33 +1017,20 @@ print_element_features (const gchar * element_name)
   GstPluginFeature *feature;
 
   /* FIXME implement other pretty print function for these */
-  feature = gst_registry_pool_find_feature (element_name,
-      GST_TYPE_SCHEDULER_FACTORY);
-  if (feature) {
-    n_print ("%s: a scheduler\n", element_name);
-    return 0;
-  }
 #ifndef GST_DISABLE_INDEX
-  feature = gst_registry_pool_find_feature (element_name,
+  feature = gst_default_registry_find_feature (element_name,
       GST_TYPE_INDEX_FACTORY);
   if (feature) {
     n_print ("%s: an index\n", element_name);
     return 0;
   }
 #endif
-  feature = gst_registry_pool_find_feature (element_name,
+  feature = gst_default_registry_find_feature (element_name,
       GST_TYPE_TYPE_FIND_FACTORY);
   if (feature) {
     n_print ("%s: a typefind function\n", element_name);
     return 0;
   }
-#ifndef GST_DISABLE_URI
-  feature = gst_registry_pool_find_feature (element_name, GST_TYPE_URI_HANDLER);
-  if (feature) {
-    n_print ("%s: an uri handler\n", element_name);
-    return 0;
-  }
-#endif
 
   return -1;
 }
@@ -1084,6 +1041,15 @@ print_element_info (GstElementFactory * factory, gboolean print_names)
   GstElement *element;
   gint maxlevel = 0;
 
+  factory =
+      GST_ELEMENT_FACTORY (gst_plugin_feature_load (GST_PLUGIN_FEATURE
+          (factory)));
+
+  if (!factory) {
+    g_print ("element plugin couldn't be loaded\n");
+    return -1;
+  }
+
   element = gst_element_factory_create (factory, NULL);
   if (!element) {
     g_print ("couldn't construct element for some reason\n");
@@ -1096,13 +1062,18 @@ print_element_info (GstElementFactory * factory, gboolean print_names)
     _name = "";
 
   print_factory_details_info (factory);
-  if (GST_PLUGIN_FEATURE (factory)->manager) {
-    GstPlugin *plugin = (GstPlugin *) GST_PLUGIN_FEATURE (factory)->manager;
+  if (GST_PLUGIN_FEATURE (factory)->plugin_name) {
+    GstPlugin *plugin;
 
-    print_plugin_info (plugin);
+    plugin = gst_registry_find_plugin (gst_registry_get_default (),
+        GST_PLUGIN_FEATURE (factory)->plugin_name);
+    if (plugin) {
+      print_plugin_info (plugin);
+    }
   }
 
   print_hierarchy (G_OBJECT_TYPE (element), 0, &maxlevel);
+  print_interfaces (G_OBJECT_TYPE (element));
 
   print_pad_templates_info (element, factory);
   print_element_flag_info (element);
@@ -1111,11 +1082,10 @@ print_element_info (GstElementFactory * factory, gboolean print_names)
   print_index_info (element);
   print_pad_info (element);
   print_element_properties_info (element);
-  print_dynamic_parameters_info (element);
   print_signal_info (element);
   print_children_info (element);
 
-  if (_name != "")
+  if (_name[0] != '\0')
     g_free (_name);
 
   return 0;
@@ -1125,20 +1095,28 @@ int
 main (int argc, char *argv[])
 {
   gboolean print_all = FALSE;
-  struct poptOption options[] = {
-    {"print-all", 'a', POPT_ARG_NONE | POPT_ARGFLAG_STRIP, &print_all, 0,
+  GOptionEntry options[] = {
+    {"print-all", 'a', 0, G_OPTION_ARG_NONE, &print_all,
         N_("Print all elements"), NULL},
-    POPT_TABLEEND
+    {NULL}
   };
+  GOptionContext *ctx;
+  GError *err = NULL;
 
-#ifdef GETTEXT_PACKAGE
+#ifdef ENABLE_NLS
   bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
   bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
   textdomain (GETTEXT_PACKAGE);
 #endif
 
-  gst_init_with_popt_table (&argc, &argv, options);
-  gst_control_init (&argc, &argv);
+  ctx = g_option_context_new ("gst-inspect");
+  g_option_context_add_main_entries (ctx, options, GETTEXT_PACKAGE);
+  g_option_context_add_group (ctx, gst_init_get_option_group ());
+  if (!g_option_context_parse (ctx, &argc, &argv, &err)) {
+    g_print ("Error initializing: %s\n", err->message);
+    exit (1);
+  }
+  g_option_context_free (ctx);
 
   if (print_all && argc > 2) {
     g_print ("-a requires no extra arguments\n");
@@ -1159,21 +1137,37 @@ main (int argc, char *argv[])
     /* if there's a factory, print out the info */
     if (factory) {
       retval = print_element_info (factory, print_all);
+      gst_object_unref (factory);
     } else {
       retval = print_element_features (arg);
     }
 
     /* otherwise check if it's a plugin */
     if (retval) {
-      plugin = gst_registry_pool_find_plugin (arg);
+      plugin = gst_default_registry_find_plugin (arg);
 
       /* if there is such a plugin, print out info */
       if (plugin) {
         print_plugin_info (plugin);
         print_plugin_features (plugin);
       } else {
-        g_print ("No such element or plugin '%s'\n", arg);
-        return -1;
+        GError *error = NULL;
+
+        if (g_file_test (arg, G_FILE_TEST_EXISTS)) {
+          plugin = gst_plugin_load_file (arg, &error);
+
+          if (plugin) {
+            print_plugin_info (plugin);
+            print_plugin_features (plugin);
+          } else {
+            g_print ("Error loading plugin file: %s\n", error->message);
+            g_error_free (error);
+            return -1;
+          }
+        } else {
+          g_print ("No such element or plugin '%s'\n", arg);
+          return -1;
+        }
       }
     }
   }
